/**
 * Copyright (C) 2012 Matteo Tamigi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.
 */
 
#include <cstring>
#include <fstream>

#include "common.hpp"
#include "PasswordGenerator.hpp"

#ifndef PDFSEC_HPP
#define PDFSEC_HPP

class PDFSec {
    public:
        //Funzioni pubbliche per la classe PDFSec (specificate in PDFSec.cpp)
        //Costruttori e distruttori
        PDFSec ();
        ~PDFSec ();
        //Funzioni per la shell da sostituire a parametri pubblici
        void PrintInfos ();
        //Funzioni primarie
        void LoadPdfInfos ();
        void initCracking ( const string );     //Gli passo la user_password (se la conosco)
        string crackUserPassword ();
        //int crypt ( string , string , unsigned );
        //int decrypt ( string , string );
        //Metodi SET e GET
        void SetFile ( const string file ) { nomeFile = file; }
        string GetFile () { return nomeFile; }
        unsigned GetDimFile () { return dimFile; }
        int GetErr () { return err; }
        int GetWrn () { return wrn; }
        EncData * GetPdfInfos ();
        //DEBUG INFOS
        bool verbose;
    private:
        //VARIABILI
        //--------------------------------------------------------------------//
        string nomeFile;
        unsigned dimFile;
        //Variabili per il parsing e il mantenimento delle info sul file
        EncData *infoPdf;        
        ifstream is;
        int err;
        int wrn;
        //Variabili per il cracking del PDF
        bool isUserPswd;
        string userPswd;
        PasswordGen pswd;
        vector<uint8_t> key;
        unsigned sizeKey;
        //--------------------------------------------------------------------//
        //METODI
        //--------------------------------------------------------------------//
        //Funzioni per la lettura del PDF (funzioni specificate in PDFParser.cpp)
        //Completate e (apparentemente) funzionanti! (rifatte da pdfcrack-0.11)
        int readPdfFile ();
        int findTrailer ();
        vector<uint8_t> parseID ( const string );
        bool getEncInfo ( const int );
        bool parseEncObject ( const string );
        vector<uint8_t> stringToByte ( const string );
        void clearPdfInfos ();
        //Funzioni per il cracking delle password del PDF (funzioni specificate in PDFCrack.cpp)
        //(rifatte da pdfcrack-0.11)
        bool isUserPasswordRev2 ();
        bool isUserPasswordRev3 ();
        vector<uint8_t> generatePaddedPassword ( char * );
        //--------------------------------------------------------------------//
};
 
 
 
 #endif
