/**
 * Copyright (C) 2012 Matteo Tamigi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.
 */

#include "PasswordGenerator.hpp"

PasswordGen :: PasswordGen () {
    EOS = true;
    state = NULL;
    charSet = NULL;
    pswd = NULL;
}

PasswordGen :: PasswordGen ( const unsigned min,
                             const unsigned max,
                             const string chs = STDCHARSET ) {
    //Azzero le variabili
    EOS = true;
    state = NULL;
    charSet = NULL;
    pswd = NULL;
    //Genero il charset
    SetCharset ( chs );
    //Genero lo stato iniziale
    SetState ( min , max , NULL );
}

PasswordGen :: ~PasswordGen () {
    if( state )
        delete [] state;
    if( charSet )
        delete [] charSet;
    if( pswd )
        delete [] pswd;
}

void PasswordGen :: SetCharset ( const string chs ) {
    delete [] charSet;
    lenSet = chs.length();
    charSet = new uint8_t[lenSet];
    for(unsigned i = 0; i < lenSet; i++)
        charSet[i] = (uint8_t)chs[i];
}

void PasswordGen :: SetState ( const unsigned min = 1, const unsigned max = 5, unsigned * st = NULL ) {
    //Verifico gli estremi di generazione, se non son validi
    //imposto gli estremi da 1 a 5 di default
    if( min > max && 1 < max && 1 < min ) { min_c = 1; max_c = 5; }
    else { max_c = max; min_c = min; }
    //Genero il vettore degli stati e lo carico (o lo azzero)
    delete [] state;
    state = new unsigned[max_c];
    if( st ) {
        //Copio lo stato
        for(unsigned i = 0; i < max_c; i++)
            state[i] = st[i];
        //Determino la lunghezza
        for(unsigned i = 0; i < max_c; i++)
            if(state[i] == -1)
                lenState = i;
    }
    else {
        for(unsigned i = 0; i < min_c; i++)
            state[i] = 0;
        for(unsigned i = min_c; i < max_c; i++)
            state[i] = -1;
        lenState = min_c;
        
    }
    //Imposto la char
    delete [] pswd;
    pswd = new char[max_c+1];
    unsigned i;
    for(i = 0; i < lenState; i++)
        pswd[i] = charSet[state[i]];
    for(i = i ; i < max_c+1; i++)
        pswd[i] = '\0';
    EOS = false;
}

void PasswordGen :: SetNextPswd () {
    //Incremento la combinazione di 1
    unsigned i = 0;
    while ( i < max_c + 1 ) {
        if( ++state[i] < lenSet ) {
            pswd[i] = charSet[state[i]];
            break;
        }
        else {
            if( i == max_c ) {
                pswd[0] = '\0';
                EOS = false;
                break;
            }
            if( i+1 == lenState ) {
                lenState++;
            }
            state[i] = 0;
            pswd[i] = charSet[0];
        }
        i++;
    }
}
