/**
 * Copyright (C) 2012 Matteo Tamigi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.
 */ 

#include <stdint.h>
#include <vector>
 
#ifndef COMMON_HPP
#define COMMON_HPP

//Definizione degli errori:
//----------------------------------------------------------------------------//
//Errori nel parsing
#define ERRNOFILE       -1      //Non esiste il file specificato
#define ERRVERS         -2      //Versione del file non supportata
#define ERRNOENCINFO    -3      //Oggetto EncInfo non trovato nel file, errore/nessuna criptazione
#define ERRNOFILEID     -4      //Trailer: FileID Object non trovato
#define ERRNOINFOTRA    -5      //Trailer non trovato
//Errori nel cracking
#define ERRUSTR         -6      //La user password  errata
//----------------------------------------------------------------------------//
//Definizione dei warning
//----------------------------------------------------------------------------//
#define WRNOSTR         -1      //Dimensione della owner_string diversa da 32 byte
#define WRNUSTR         -2      //Dimensione della user_string diversa da 32 byte    
//----------------------------------------------------------------------------//

#define STDCHARSET "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
#define STD_EXT_CHARSET "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890.,:;-_<>@#+*[]^'?=)(/&%$\"!|\\"

/** 
    EncData holds all the information regarding the encryption-setting of a 
    specific pdf. 
    s_handler - Security handler string.
    o_string - Owner-string, 32 bytes, not null-terminated
    u_string - User-string, 32 bytes, not null-terminated
    fileID - file ID in fileIDLen bytes, not null-terminated
*/
struct EncData {
    string s_handler;
    vector<uint8_t> o_string;
    vector<uint8_t> u_string;
    vector<uint8_t> fileID;
    bool encryptMetaData;
    //unsigned int fileIDLen;
    unsigned int version_major;
    unsigned int version_minor;
    int length;
    int permissions;
    int revision;
    int version;
};

//Strutture relative al parsing del file PDF
struct p_str {
  uint8_t *content;
  uint8_t len;
};

#endif
